@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -28,12 +28,21 @@
 #include "libnormaliz/nmz_nauty.h"
 #include "libnormaliz/cone.h"
 #include "libnormaliz/full_cone.h"
-// #include "libnormaliz/map_operations.h"
+#include "libnormaliz/list_and_map_operations.h"
+#include "libnormaliz/nmz_hash.h"
 
 namespace libnormaliz {
 
 using namespace std;
 
+// meant for a posteriori changes of GensRef
+// for example, when a coordinate transformation has been applied
+// and we want the GensRef in their original coordinates
+template <typename Integer>
+void AutomorphismGroup<Integer>::setGensRef(const Matrix<Integer>& GivenGensRef) {
+    GensRef = GivenGensRef;
+}
+
 /* Unused getters
 template <typename Integer>
 AutomParam::Method AutomorphismGroup<Integer>::getMethod() const {
@@ -43,6 +52,31 @@ AutomParam::Method AutomorphismGroup<Integer>::getMethod() const {
 template <typename Integer>
 bool AutomorphismGroup<Integer>::Is_Computed(AutomParam::Goals goal) const {
     return contains(is_Computed, goal);
+} */
+
+template <typename Integer>
+bool AutomorphismGroup<Integer>::HasQuality(AutomParam::Quality quality) const {
+    return getQualitiesString().find(quality_to_string(quality)) != string::npos;
+}
+
+template <typename Integer>
+bool AutomorphismGroup<Integer>::IsIntegral() const {
+    return is_integral;
+}
+
+template <typename Integer>
+bool AutomorphismGroup<Integer>::IsInput() const {
+    return HasQuality(AutomParam::input_gen) || HasQuality(AutomParam::input_ineq);
+}
+
+template <typename Integer>
+bool AutomorphismGroup<Integer>::IsAmbient() const {
+    return HasQuality(AutomParam::ambient_gen) || HasQuality(AutomParam::ambient_ineq);
+}
+
+template <typename Integer>
+bool AutomorphismGroup<Integer>::IsIntegralityChecked() const {
+    return integrality_checked;
 }
 
 template <typename Integer>
@@ -60,44 +94,72 @@ const Matrix<Integer>& AutomorphismGroup<Integer>::getLinForms() const {
     return LinFormsRef;
 }
 
+/*
+
 template <typename Integer>
 const Matrix<Integer>& AutomorphismGroup<Integer>::getSpecialLinForms() const {
     return SpecialLinFormsRef;
 }
 */
 
-template <typename Integer>
-const vector<vector<key_t> >& AutomorphismGroup<Integer>::getExtremeRaysPerms() const {
-    return ExtRaysPerms;
-}
-
-template <typename Integer>
-const vector<vector<key_t> >& AutomorphismGroup<Integer>::getVerticesPerms() const {
-    return VerticesPerms;
-}
-
 template <typename Integer>
 mpz_class AutomorphismGroup<Integer>::getOrder() const {
     return order;
 }
 
+template <typename Integer>
+const vector<vector<key_t> >& AutomorphismGroup<Integer>::getGensPerms() const {
+    return GenPerms;
+}
+
+template <typename Integer>
+const vector<vector<key_t> >& AutomorphismGroup<Integer>::getLinFormsPerms() const {
+    return LinFormPerms;
+}
+
+template <typename Integer>
+const vector<vector<key_t> >& AutomorphismGroup<Integer>::getGensOrbits() const {
+    return GenOrbits;
+}
+
+template <typename Integer>
+const vector<vector<key_t> >& AutomorphismGroup<Integer>::getLinFormsOrbits() const {
+    return LinFormOrbits;
+}
+
+template <typename Integer>
+const vector<vector<key_t> >& AutomorphismGroup<Integer>::getExtremeRaysPerms() const {
+    assert(cone_dependent_data_computed);
+    return ExtRaysPerms;
+}
+
+template <typename Integer>
+const vector<vector<key_t> >& AutomorphismGroup<Integer>::getVerticesPerms() const {
+    assert(cone_dependent_data_computed);
+    return VerticesPerms;
+}
+
 template <typename Integer>
 const vector<vector<key_t> >& AutomorphismGroup<Integer>::getSupportHyperplanesPerms() const {
+    assert(cone_dependent_data_computed);
     return SuppHypsPerms;
 }
 
 template <typename Integer>
 const vector<vector<key_t> >& AutomorphismGroup<Integer>::getExtremeRaysOrbits() const {
+    assert(cone_dependent_data_computed);
     return ExtRaysOrbits;
 }
 
 template <typename Integer>
 const vector<vector<key_t> >& AutomorphismGroup<Integer>::getVerticesOrbits() const {
+    assert(cone_dependent_data_computed);
     return VerticesOrbits;
 }
 
 template <typename Integer>
 const vector<vector<key_t> >& AutomorphismGroup<Integer>::getSupportHyperplanesOrbits() const {
+    assert(cone_dependent_data_computed);
     return SuppHypsOrbits;
 }
 
@@ -112,17 +174,20 @@ const vector<key_t>& AutomorphismGroup<Integer>::getCanLabellingGens() const {
     return CanLabellingGens;
 }
 
-template <typename Integer>
-const BinaryMatrix<Integer>& AutomorphismGroup<Integer>::getCanType() const{
-    return CanType;
-}
-
 */
 
+template <typename Integer>
+const BinaryMatrix<Integer>& AutomorphismGroup<Integer>::getCanType() const {
+    return CanType;
+}
+
 template <typename Integer>
 void AutomorphismGroup<Integer>::reset() {
-    
     order = 1;
+    makeCanType = false;
+    cone_dependent_data_computed = false;
+    is_integral = false;
+    integrality_checked = false;
 }
 
 template <typename Integer>
@@ -135,6 +200,7 @@ template<typename Integer>
 AutomorphismGroup<Integer>::AutomorphismGroup(const Matrix<Integer>& ExtRays, const Matrix<Integer>& SpecialGens,
         const Matrix<Integer>& SupHyps,  const Matrix<Integer>& SpecialLinearForms){
 
+    reset();
     method=AutomParam::E;
 
     Gens=ExtRays; // reference for orbits
@@ -211,13 +277,30 @@ string quality_to_string(AutomParam::Quality quality) {
         return "Integral";
     if (quality == AutomParam::euclidean)
         return "Euclidean";
-    if (quality == AutomParam::ambient)
-        return "Ambient";
+    if (quality == AutomParam::ambient_gen)
+        return "Ambient(from generators)";
+    if (quality == AutomParam::ambient_ineq)
+        return "Ambient(from inequalities)";
+    if (quality == AutomParam::input_gen)
+        return "Input(from generators)";
+    if (quality == AutomParam::input_ineq)
+        return "Input(from inequalities)";
     if (quality == AutomParam::algebraic)
         return "Algebraic";
     if (quality == AutomParam::graded)
         return "Graded";
+    if (quality == AutomParam::monoid)
+        return "Monoid";
     assert(false);
+    return string();  // silence compiler warning
+}
+
+template <typename Integer>
+void AutomorphismGroup<Integer>::fromInputToMonoid(){
+
+    if(Qualities.find(AutomParam::input_gen) != Qualities.end())
+        Qualities.erase(AutomParam::input_gen);
+    Qualities.insert(AutomParam::monoid);
 }
 
 template <typename Integer>
@@ -228,24 +311,32 @@ string AutomorphismGroup<Integer>::getQualitiesString() const {
     return result;
 }
 
-/* unused constructor 
 template <typename Integer>
 AutomorphismGroup<Integer>::AutomorphismGroup(const Matrix<Integer>& ExtRays,
                                               const Matrix<Integer>& SpecialGens,
                                               const Matrix<Integer>& SuppHyps,
                                               const Matrix<Integer>& SpecialLinForms) {
+    reset();
     set_basic_gens_and_lin_forms(ExtRays, SpecialGens, SuppHyps, SpecialLinForms);
 }
-*/
+
+template <typename Integer>
+void AutomorphismGroup<Integer>::activateCanType(bool onoff) {
+    makeCanType = onoff;
+}
 
 template <typename Integer>
 AutomorphismGroup<Integer>::AutomorphismGroup(const Matrix<Integer>& ExtRays,
                                               const Matrix<Integer>& SuppHyps,
                                               const Matrix<Integer>& SpecialLinForms) {
+
+    // SuppHyps.debug_print('+');
+    // ExtRays.debug_print('$');
+    reset();
     size_t dim = ExtRays.nr_of_columns();
     Matrix<Integer> SpecialGens(0, dim);
     set_basic_gens_and_lin_forms(ExtRays, SpecialGens, SuppHyps, SpecialLinForms);
-    if(ExtRays.nr_of_rows() == 0)
+    if (ExtRays.nr_of_rows() == 0)
         order = 1;
 }
 
@@ -254,6 +345,7 @@ void AutomorphismGroup<Integer>::set_basic_gens_and_lin_forms(const Matrix<Integ
                                                               const Matrix<Integer>& SpecialGens,
                                                               const Matrix<Integer>& SuppHyps,
                                                               const Matrix<Integer>& SpecialLinForms) {
+    reset();
     GensRef = ExtRays;  // reference data
     LinFormsRef = SuppHyps;
     SpecialLinFormsRef = SpecialLinForms;
@@ -262,11 +354,6 @@ void AutomorphismGroup<Integer>::set_basic_gens_and_lin_forms(const Matrix<Integ
     nr_special_linforms = SpecialLinForms.nr_of_rows();
     nr_special_gens = SpecialGens.nr_of_rows();
 
-    GensComp = GensRef;
-    GensComp.append(SpecialGensRef);
-    LinFormsComp = LinFormsRef;
-    LinFormsComp.append(SpecialLinFormsRef);
-
     addedComputationGens = false;
     addedComputationLinForms = false;
 }
@@ -281,7 +368,6 @@ void AutomorphismGroup<Integer>::addComputationGens(const Matrix<Integer>& Given
     addedComputationGens = true;
 }
 
-
 /*
 template <typename Integer>
 void AutomorphismGroup<Integer>::addComputationLinForms(const Matrix<Integer>& GivenLinearForms) {
@@ -302,6 +388,7 @@ void AutomorphismGroup<Integer>::dualize() {
     swap(addedComputationGens, addedComputationLinForms);
 }
 
+// contravariant -- swaps only computation results !!
 template <typename Integer>
 void AutomorphismGroup<Integer>::swap_data_from_dual(AutomorphismGroup<Integer> Dual) {
     swap(GenPerms, Dual.LinFormPerms);
@@ -315,9 +402,14 @@ void AutomorphismGroup<Integer>::swap_data_from_dual(AutomorphismGroup<Integer>
     }
 
     order = Dual.order;
+    is_integral = Dual.is_integral;
+    integrality_checked = Dual.integrality_checked;
     Qualities = Dual.Qualities;
+
+    // Note: CanType cannot be dualized
 }
 
+// covariant  -- swaps only computation results !!
 template <typename Integer>
 void AutomorphismGroup<Integer>::swap_data_from(AutomorphismGroup<Integer> Help) {
     swap(GenPerms, Help.GenPerms);
@@ -329,7 +421,10 @@ void AutomorphismGroup<Integer>::swap_data_from(AutomorphismGroup<Integer> Help)
         LinMaps.push_back(Help.LinMaps[i]);
     }
 
+    CanType = Help.CanType;  // no swap yet ...
     order = Help.order;
+    is_integral = Help.is_integral;
+    integrality_checked = Help.integrality_checked;
     Qualities = Help.Qualities;
 }
 
@@ -399,13 +494,21 @@ bool AutomorphismGroup<Integer>::compute_integral() {
     bool success = false;
     bool gens_tried = false;
 
-    if (addedComputationGens || GensComp.nr_of_rows() <= LinFormsComp.nr_of_rows() || LinFormsRef.nr_of_rows() == 0) {
+    size_t nr_gens_used = GensComp.nr_of_rows();
+    if (nr_gens_used == 0)
+        nr_gens_used = GensRef.nr_of_rows();
+
+    size_t nr_linforms_used = LinFormsComp.nr_of_rows();
+    if (nr_linforms_used == 0)
+        nr_linforms_used = LinFormsRef.nr_of_rows();
+
+    if (addedComputationGens || nr_gens_used <= nr_linforms_used || nr_linforms_used == 0 || makeCanType) {
         success = compute_inner(AutomParam::integral);
         gens_tried = true;
     }
 
-    if (success)
-        return true;
+    if (success || makeCanType)  // if the CanType is asked for, dualization is not aloowed
+        return success;
 
     AutomorphismGroup<Integer> Dual(*this);
     Dual.dualize();
@@ -440,13 +543,77 @@ bool AutomorphismGroup<Integer>::compute(const AutomParam::Quality& desired_qual
     return compute_inner(desired_quality, force_gens_x_linforms);
 }
 
+template <typename Integer>
+nauty_result<Integer> AutomorphismGroup<Integer>::prepare_Gns_only_and_apply_nauty(const AutomParam::Quality& desired_quality) {
+    if (nr_special_gens == 0 && !addedComputationGens) {
+#ifdef NMZ_NAUTY
+        return compute_automs_by_nauty_FromGensOnly(GensRef, nr_special_gens, SpecialLinFormsRef, desired_quality);
+#else
+        throw NotComputableException("Automorphism groups and iso types not accessible without nauty");
+#endif
+    }
+    else {
+        if (!addedComputationGens)
+            GensComp = GensRef;
+        GensComp.append(SpecialGensRef);
+#ifdef NMZ_NAUTY
+        return compute_automs_by_nauty_FromGensOnly(GensComp, nr_special_gens, SpecialLinFormsRef, desired_quality);
+#else
+        throw NotComputableException("Automorphism groups and iso types not accessible without nauty");
+#endif
+    }
+}
+
+template <typename Integer>
+nauty_result<Integer> AutomorphismGroup<Integer>::prepare_Gns_x_LF_only_and_apply_nauty(
+    const AutomParam::Quality& desired_quality) {
+    // cout << "**** " << addedComputationGens << " " << addedComputationLinForms << " " << GensComp.nr_of_rows() << " " <<
+    // LinFormsComp.nr_of_rows() << endl;
+
+    if (nr_special_gens > 0 || addedComputationGens) {
+        if (!addedComputationGens) {
+            GensComp = GensRef;
+        }
+        GensComp.append(SpecialGensRef);
+    }
+
+    if (nr_special_linforms > 0 || addedComputationLinForms) {
+        if (!addedComputationLinForms) {
+            LinFormsComp = LinFormsRef;
+        }
+        LinFormsComp.append(SpecialLinFormsRef);
+    }
+
+    // cout << "**** " << addedComputationGens << " " << addedComputationLinForms << " " << GensComp.nr_of_rows() << " " <<
+    // LinFormsComp.nr_of_rows() << endl;
+
+#ifdef NMZ_NAUTY
+    if (GensComp.nr_of_rows() == 0) {
+        if (LinFormsComp.nr_of_rows() == 0)
+            return compute_automs_by_nauty_Gens_LF(GensRef, nr_special_gens, LinFormsRef, nr_special_linforms, desired_quality);
+        else
+            return compute_automs_by_nauty_Gens_LF(GensRef, nr_special_gens, LinFormsComp, nr_special_linforms, desired_quality);
+    }
+    else {
+        if (LinFormsComp.nr_of_rows() == 0)
+            return compute_automs_by_nauty_Gens_LF(GensComp, nr_special_gens, LinFormsRef, nr_special_linforms, desired_quality);
+        else
+            return compute_automs_by_nauty_Gens_LF(GensComp, nr_special_gens, LinFormsComp, nr_special_linforms, desired_quality);
+    }
+#else
+    throw NotComputableException("Automorphism groups and iso types not accessible without nauty");
+#endif
+}
+
 template <typename Integer>
 bool AutomorphismGroup<Integer>::compute_inner(const AutomParam::Quality& desired_quality, bool force_gens_x_linforms) {
     bool FromGensOnly = true;
-    if (desired_quality == AutomParam::combinatorial || desired_quality == AutomParam::ambient || force_gens_x_linforms)
+    if (desired_quality == AutomParam::combinatorial || desired_quality == AutomParam::ambient_gen ||
+        desired_quality == AutomParam::ambient_ineq || force_gens_x_linforms)
         FromGensOnly = false;
 
     assert(desired_quality == AutomParam::integral || !addedComputationGens);
+    assert(!makeCanType || desired_quality == AutomParam::integral || desired_quality == AutomParam::rational);
 
     if (!FromGensOnly) {
         if (!addedComputationGens) {
@@ -474,44 +641,52 @@ bool AutomorphismGroup<Integer>::compute_inner(const AutomParam::Quality& desire
 
 #ifdef NMZ_NAUTY
     if (FromGensOnly) {
-        result = compute_automs_by_nauty_FromGensOnly(GensComp, nr_special_gens, SpecialLinFormsRef, desired_quality);
+        result = prepare_Gns_only_and_apply_nauty(desired_quality);
     }
     else {
-        result = compute_automs_by_nauty_Gens_LF(GensComp, nr_special_gens, LinFormsComp, nr_special_linforms, desired_quality);
+        result = prepare_Gns_x_LF_only_and_apply_nauty(desired_quality);
     }
 #endif
 
     order = result.order;
-    CanType = result.CanType;
+    if (makeCanType)
+        CanType = result.CanType;
 
-    bool maps_lifted = false;
-    if (desired_quality != AutomParam::combinatorial && desired_quality != AutomParam::euclidean) {
-        maps_lifted = make_linear_maps_primal(GensComp, result.GenPerms);
+    Qualities.insert(desired_quality);
+
+    if (!using_renf<Integer>() && (HasQuality(AutomParam::ambient_gen) || HasQuality(AutomParam::ambient_ineq))) {
+        is_integral = true;
+        integrality_checked = true;
+    }
+
+    bool check_integrality = false;  // the critical point in this case is that full dimension may be reached only
+    if (!using_renf<Integer>() && HasQuality(AutomParam::input_ineq)) {  // with the dehomogenization which is a special genarator
+        size_t gens_ref_rank = GensRef.rank();                           // i.e., a fixed point in this setting
+        if (GensRef.nr_of_rows() > 0 && gens_ref_rank == GensRef[0].size())
+            check_integrality = true;
+    }
+
+    if (HasQuality(AutomParam::integral) ||
+        HasQuality(AutomParam::rational) ||  // in the algebraic case we compute the linear maps
+        HasQuality(AutomParam::algebraic) || HasQuality(AutomParam::input_gen) || check_integrality) {
+        integrality_checked = true;
+        if (GensComp.nr_of_rows() > 0)
+            is_integral = make_linear_maps_primal(GensComp, result.GenPerms);
+        else
+            is_integral = make_linear_maps_primal(GensRef, result.GenPerms);
     }
 
     // cout << "LLLL " << maps_lifted << endl;
 
-    if (!maps_lifted && desired_quality == AutomParam::integral)
+    if (!is_integral && desired_quality == AutomParam::integral)
         return false;
 
-    // cout << quality_to_string(desired_quality) << " " << maps_lifted << endl;
+    if (using_renf<Integer>()) {  // makes no sense in this case
+        is_integral = false;
+        integrality_checked = false;
+    }
 
-    if (maps_lifted) {
-        if (desired_quality == AutomParam::ambient)
-            Qualities.insert(AutomParam::ambient);
-        if (desired_quality == AutomParam::integral)
-            Qualities.insert(AutomParam::integral);
-        if (desired_quality == AutomParam::rational)
-            Qualities.insert(AutomParam::integral);
-        if (desired_quality == AutomParam::algebraic)
-            Qualities.insert(AutomParam::algebraic);
-    }
-    else {
-        if (desired_quality == AutomParam::rational)
-            Qualities.insert(AutomParam::rational);
-        else
-            Qualities.insert(desired_quality);
-    }
+    // cout << quality_to_string(desired_quality) << " " << maps_lifted << endl;
 
     if (true) {  //(contains(ToCompute,AutomParam::OrbitsPrimal)){
         if (method == AutomParam::EH || method == AutomParam::EL || method == AutomParam::EE) {
@@ -539,7 +714,18 @@ bool AutomorphismGroup<Integer>::compute_inner(const AutomParam::Quality& desire
     /* CanLabellingGens.clear();
     if(!addedComputationGens){
         CanLabellingGens=result.CanLabellingGens;
-    }*/
+    }
+    cout << "===========" << endl;
+    cout << result.GenPerms;
+    cout << "===========" << endl;
+    cout << GenPerms;
+    cout << "===========" << endl;
+    cout << LinFormPerms;
+    cout << "===========" << endl;
+    cout << GenOrbits;
+    cout << "===========" << endl;
+    cout << LinFormOrbits;
+    cout << "===========" << endl;*/
 
     return true;
 }
@@ -592,17 +778,30 @@ void AutomorphismGroup<Integer>::linform_data_via_lin_maps() {
 */
 
 template <typename Integer>
-void AutomorphismGroup<Integer>::linform_data_via_incidence() {
-    map<dynamic_bitset, int> IncidenceMap;
+void AutomorphismGroup<Integer>::setIncidenceMap(const map<dynamic_bitset, key_t>& Incidence) {
+    IncidenceMap = Incidence;
+    assert(IncidenceMap.size() == LinFormsRef.nr_of_rows());
+    if (IncidenceMap.size() > 0)
+        assert(IncidenceMap.begin()->first.size() == GensRef.nr_of_rows());
+}
 
-    for (size_t i = 0; i < LinFormsRef.nr_of_rows(); ++i) {
-        dynamic_bitset indicator(GensRef.nr_of_rows());
-        for (size_t j = 0; j < GensRef.nr_of_rows(); ++j) {
-            if (v_scalar_product(LinFormsRef[i], GensRef[j]) == 0)
-                indicator[j] = 1;
-        }
-        IncidenceMap[indicator] = i;
-    }
+template <typename Integer>
+void AutomorphismGroup<Integer>::compute_incidence_map() {
+    if (IncidenceMap.size() > 0)  // already computed or set from the outside
+        return;
+
+    vector<dynamic_bitset> IncidenceMatrix;
+
+    makeIncidenceMatrix(IncidenceMatrix, GensRef, LinFormsRef);
+    IncidenceMap = map_vector_to_indices(IncidenceMatrix);
+    // cout << "IIIIIIIIII " << IncidenceMap.size() << "--  " << LinFormsRef.nr_of_rows() <<  "--  " << GensRef.nr_of_rows() <<
+    // endl;
+    assert(IncidenceMap.size() == LinFormsRef.nr_of_rows());
+}
+
+template <typename Integer>
+void AutomorphismGroup<Integer>::linform_data_via_incidence() {
+    compute_incidence_map();
 
     LinFormPerms.clear();
     LinFormPerms.resize(GenPerms.size());
@@ -620,7 +819,6 @@ void AutomorphismGroup<Integer>::linform_data_via_incidence() {
     LinFormOrbits = orbits(LinFormPerms, LinFormsRef.nr_of_rows());
 }
 
-/*
 // the next two functions create the orbit of a vector from the action of linear maps
 template <typename Integer>
 void AutomorphismGroup<Integer>::add_images_to_orbit(const vector<Integer>& v, set<vector<Integer> >& orbit) const {
@@ -645,7 +843,6 @@ list<vector<Integer> > AutomorphismGroup<Integer>::orbit_primal(const vector<Int
         orbit_list.push_back(c);
     return orbit_list;
 }
-*/
 
 //-------------------------------------------------------------------------------
 
@@ -680,7 +877,6 @@ IsoType<Integer>::IsoType(Full_Cone<Integer>& C, bool with_Hilbert_basis){
 
 template <typename Integer>
 IsoType<Integer>::IsoType() {  // constructs a dummy object
-
 }
 
 /*
@@ -734,64 +930,153 @@ IsoType<Integer>::IsoType(const Full_Cone<Integer>& C, bool& success) {
 }
 */
 
-
 template <typename Integer>
 IsoType<Integer>::IsoType(Cone<Integer>& C) {
-    
-    quality = AutomParam::integral; // for tihe time being
+    type = AutomParam::integral_standard;
 
     C.compute(ConeProperty::HilbertBasis);
-    
+
     /* cout << "****************" << endl;
     C.getHilbertBasisMatrix().pretty_print(cout);
     cout << "----------------" << endl;
     C.getSupportHyperplanesMatrix().pretty_print(cout);
     cout << "****************" << endl; */
-    
-    Matrix<Integer> HB_sublattice=C.getSublattice().to_sublattice(C.getHilbertBasis());
-    Matrix<Integer> SH_sublattice=C.getSublattice().to_sublattice_dual(C.getSupportHyperplanes());
-    
+
+    Matrix<Integer> HB_sublattice = C.getSublattice().to_sublattice(C.getHilbertBasis());
+    Matrix<Integer> SH_sublattice = C.getSublattice().to_sublattice_dual(C.getSupportHyperplanes());
+
     /* HB_sublattice.pretty_print(cout);
     cout << "----------------" << endl;
     SH_sublattice.pretty_print(cout);
     cout << "****************" << endl; */
 
 #ifndef NMZ_NAUTY
-    
-    throw FatalException("IsoType neds nauty");
-    
+
+    throw FatalException("IsoType needs nauty");
 #else
-    
-    nauty_result<Integer> nau_res = compute_automs_by_nauty_Gens_LF(HB_sublattice,0, SH_sublattice,
-                                                        0, quality);
+
+    nauty_result<Integer> nau_res = compute_automs_by_nauty_Gens_LF(HB_sublattice, 0, SH_sublattice, 0, AutomParam::integral);
     CanType = nau_res.CanType;
-#endif 
-    
+#endif
 }
 
 template <typename Integer>
-IsoType<Integer>::IsoType(Matrix<Integer>& M) {
-    
-    quality = AutomParam::integral; // for tihe time being
-    
+IsoType<Integer>::IsoType(const Matrix<Integer>& M) {
+    type = AutomParam::matrix;  // for tihe time being
+
     Matrix<Integer> UnitMatrix(M.nr_of_columns());
-    
+
 #ifndef NMZ_NAUTY
-    
-    throw FatalException("IsoType neds nauty");
-    
+
+    throw FatalException("IsoType needs nauty");
+
 #else
-    
-    nauty_result<Integer> nau_res = compute_automs_by_nauty_Gens_LF(M,0, UnitMatrix,
-                                                        0, quality);
+
+    nauty_result<Integer> nau_res =
+        compute_automs_by_nauty_Gens_LF(M, 0, UnitMatrix, 0, AutomParam::integral);  // true = with iso type
     CanType = nau_res.CanType;
-#endif 
-    
+#endif
+}
+
+template <typename Integer>
+IsoType<Integer>::IsoType(const Matrix<Integer>& Inequalities,
+                          const Matrix<Integer> Equations,
+                          const vector<Integer> Grading,
+                          bool strict_type_check) {
+    type = AutomParam::rational_dual;
+
+    Matrix<Integer> Subspace = Equations.kernel();
+    Matrix<Integer> IneqOnSubspace(Inequalities.nr_of_rows(), Subspace.nr_of_rows());
+    for (size_t i = 0; i < Inequalities.nr_of_rows(); ++i)
+        IneqOnSubspace[i] = Subspace.MxV(Inequalities[i]);
+
+    vector<Integer> GradingOnSubspace = Subspace.MxV(Grading);
+    IneqOnSubspace.append(GradingOnSubspace);  // better to treat it as a special generator ?
+
+    /*cout << "***************" << endl;
+    IneqOnSubspace.pretty_print(cout);
+    cout << "**************" << endl;*/
+
+    Matrix<Integer> Empty(0, Subspace.nr_of_rows());
+
+#ifndef NMZ_NAUTY
+
+    throw FatalException("IsoType needs nauty");
+
+#else
+
+    nauty_result<Integer> nau_res;
+    // #pragma omp critical(NAUTY)
+    nau_res = compute_automs_by_nauty_FromGensOnly(IneqOnSubspace, 0, Empty, AutomParam::integral);
+    if (strict_type_check)
+        CanType = nau_res.CanType;
+    else {
+        ostringstream TypeStream;
+        nau_res.CanType.pretty_print(TypeStream);
+        HashValue = sha256hexvec(TypeStream.str());
+    }
+
+    /* vector<vector<key_t> > OrbitKeys = convert_to_orbits(nau_res.GenOrbits);
+    FacetOrbits.clear();
+    for(size_t i =0; i< OrbitKeys.size() -1; ++i)  // don't want the orbit of the grading
+        FacetOrbits.push_back(key_to_bitset(OrbitKeys[i], Inequalities.nr_of_rows()) );    */
+
+    // cout << "-----------------------------------------" << endl;
+    // cout << FacetOrbits;
+#endif
+
+    index = IneqOnSubspace.full_rank_index();
+}
+
+template <typename Integer>
+IsoType<Integer>::IsoType(const Matrix<Integer>& ExtremeRays, const vector<Integer> Grading, bool strict_type_check) {
+    type = AutomParam::rational_primal;
+
+    /*cout << "***************" << endl;
+    IneqOnSubspace.pretty_print(cout);
+    cout << "**************" << endl;*/
+
+    Sublattice_Representation<Integer> Subspace(ExtremeRays, true, false);  //  take saturation, no LLL
+    Matrix<Integer> EmbeddedExtRays = Subspace.to_sublattice(ExtremeRays);
+    vector<Integer> RestrictedGrad = Subspace.to_sublattice_dual_no_div(Grading);
+
+    Matrix<Integer> GradMat(RestrictedGrad);
+
+    // Matrix<Integer> Empty(0,Subspace.getRank());
+
+    nauty_result<Integer> nau_res;
+
+#ifndef NMZ_NAUTY
+
+    throw FatalException("IsoType needs nauty");
+
+#else
+
+#ifndef NMZ_NAUTY_TLS
+#pragma omp critical(NAUTY)
+#endif
+    nau_res = compute_automs_by_nauty_FromGensOnly(EmbeddedExtRays, 0, GradMat, AutomParam::integral);
+
+    if (strict_type_check)
+        CanType = nau_res.CanType;
+    else {
+        ostringstream TypeStream;
+        nau_res.CanType.pretty_print(TypeStream);
+        HashValue = sha256hexvec(TypeStream.str());
+    }
+#endif
+
+    // vector<vector<key_t> > OrbitKeys = convert_to_orbits(nau_res.GenOrbits);
+    // FacetOrbits.clear();
+
+    // cout << "-----------------------------------------" << endl;
+    // cout << FacetOrbits;
+
+    index = convertTo<Integer>(Subspace.getExternalIndex());
 }
 
 template <>
 IsoType<renf_elem_class>::IsoType(Cone<renf_elem_class>& C) {
-    
     assert(false);
 }
 
@@ -827,7 +1112,7 @@ mpq_class IsoType<Integer>::getMultiplicity() const {
 */
 
 template <typename Integer>
-const BinaryMatrix<Integer>& IsoType<Integer>::getCanType() const{
+const BinaryMatrix<Integer>& IsoType<Integer>::getCanType() const {
     return CanType;
 }
 
@@ -836,65 +1121,74 @@ const BinaryMatrix<Integer>& IsoType<Integer>::getCanType() const{
 template <typename Integer>
 Isomorphism_Classes<Integer>::Isomorphism_Classes() {
     // Classes.push_back(IsoType<Integer>());
+    type = AutomParam::integral_standard;
 }
 
 template <typename Integer>
-size_t Isomorphism_Classes<Integer>::size() const{
-    
+Isomorphism_Classes<Integer>::Isomorphism_Classes(AutomParam::Type given_type) {
+    // Classes.push_back(IsoType<Integer>());
+    type = given_type;
+}
+
+template <typename Integer>
+size_t Isomorphism_Classes<Integer>::size() const {
     return Classes.size();
 }
 
 template <typename Integer>
-const IsoType<Integer>& Isomorphism_Classes<Integer>::find_type(const IsoType<Integer>& IT, bool& found) const{
+const set<IsoType<Integer>, IsoType_compare<Integer> >& Isomorphism_Classes<Integer>::getClasses() const {
+    return Classes;
+}
 
-    auto F=Classes.find(IT);
-    found=true;
-    if(F==Classes.end())
-        found=false;
+template <typename Integer>
+const IsoType<Integer>& Isomorphism_Classes<Integer>::find_type(const IsoType<Integer>& IT, bool& found) const {
+    assert(IT.type == type);
+
+    auto F = Classes.find(IT);
+    found = true;
+    if (F == Classes.end())
+        found = false;
     return *F;
 }
 
 template <typename Integer>
-const IsoType<Integer>& Isomorphism_Classes<Integer>::add_type(const IsoType<Integer>& IT, bool& found){
+const IsoType<Integer>& Isomorphism_Classes<Integer>::add_type(const IsoType<Integer>& IT, bool& found) {
+    assert(IT.type == type);
 
     // typename set<IsoType<Integer>, IsoType_compare<Integer> >::iterator ICL;
-    pair < typename set<IsoType<Integer>, IsoType_compare<Integer> >::iterator , bool > ret;
-    ret= Classes.insert(IT);   
-    found=!ret.second;
+    pair<typename set<IsoType<Integer>, IsoType_compare<Integer> >::iterator, bool> ret;
+    ret = Classes.insert(IT);
+    found = !ret.second;
     /* if(!found){
-        cout << "new isoclass CanType, format " << IT.CanType.get_nr_rows()<< "x" << IT.CanType.get_nr_columns()<< endl;    
+        cout << "new isoclass CanType, format " << IT.CanType.get_nr_rows()<< "x" << IT.CanType.get_nr_columns()<< endl;
         IT.CanType.get_value_mat().pretty_print(cout);
         cout << "Values " << IT.CanType.get_values();
     }*/
-    
+
     return *ret.first;
 }
 
 template <typename Integer>
-size_t Isomorphism_Classes<Integer>::erase_type(const IsoType<Integer>& IT){
-    
-    return Classes.erase(IT);    
+size_t Isomorphism_Classes<Integer>::erase_type(const IsoType<Integer>& IT) {
+    return Classes.erase(IT);
 }
 
 template <typename Integer>
-const IsoType<Integer>& Isomorphism_Classes<Integer>::find_type(Cone<Integer>& C, bool& found) const{
-    
+const IsoType<Integer>& Isomorphism_Classes<Integer>::find_type(Cone<Integer>& C, bool& found) const {
     IsoType<Integer> IT(C);
-    return find_type(IT,found);
+    return find_type(IT, found);
 }
 
 template <typename Integer>
-const IsoType<Integer>& Isomorphism_Classes<Integer>::add_type(Cone<Integer>& C, bool& found){
-    
+const IsoType<Integer>& Isomorphism_Classes<Integer>::add_type(Cone<Integer>& C, bool& found) {
     IsoType<Integer> IT(C);
-    return add_type(IT,found);
+    return add_type(IT, found);
 }
 
 template <typename Integer>
-size_t Isomorphism_Classes<Integer>::erase_type(Cone<Integer>& C){
-
+size_t Isomorphism_Classes<Integer>::erase_type(Cone<Integer>& C) {
     IsoType<Integer> IT(C);
-    return erase_type(IT);    
+    return erase_type(IT);
 }
 
 /*
@@ -930,7 +1224,7 @@ const IsoType<Integer>& Isomorphism_Classes<Integer>::find_type(Full_Cone<Intege
 }
 */
 
-/* 
+/*
  //  old functions used for the computation of orbits
 list<dynamic_bitset> partition(size_t n, const vector<vector<key_t> >& Orbits) {
     // produces a list of bitsets, namely the indicator vectors of the key vectors in Orbits
@@ -989,49 +1283,49 @@ list<dynamic_bitset> join_partitions(const list<dynamic_bitset>& P1, const list<
 */
 
 vector<vector<key_t> > PermGroup(const vector<vector<key_t> >& Perms, size_t N) {
-// creates the full permutation group of 0,...,N-1 generated vy Perms
-    
+    // creates the full permutation group of 0,...,N-1 generated by Perms
+
     set<vector<key_t> > Group, Work;
-    
+
     Group.insert(identity_key(N));
-    for(size_t i=0;i< Perms.size(); ++i)
-        Work.insert(Perms[i]);  
-    
-    while(!Work.empty()){
+    for (size_t i = 0; i < Perms.size(); ++i)
+        Work.insert(Perms[i]);
+
+    while (!Work.empty()) {
         set<vector<key_t> > NewPerms;
-        for (auto& W: Work){
-            for(size_t j=0;j<Perms.size();++j){
+        for (auto& W : Work) {
+            for (size_t j = 0; j < Perms.size(); ++j) {
                 vector<key_t> new_perm(N);
-                for(size_t k=0;k<N;++k)
-                    new_perm[k]=Perms[j][W[k]];
-                auto p=Group.find(new_perm);
-                if(p!=Group.end())
+                for (size_t k = 0; k < N; ++k)
+                    new_perm[k] = Perms[j][W[k]];
+                auto p = Group.find(new_perm);
+                if (p != Group.end())
                     continue;
-                p=Work.find(new_perm);
-                if(p!=Work.end())
+                p = Work.find(new_perm);
+                if (p != Work.end())
                     continue;
-                NewPerms.insert(new_perm);         
-            }         
+                NewPerms.insert(new_perm);
+            }
         }
-        Group.insert(Work.begin(),Work.end());
-        Work=NewPerms;
+        Group.insert(Work.begin(), Work.end());
+        Work = NewPerms;
     }
-    
-    vector<vector<key_t> >  GroupVector;
-    for (auto& W: Group)
+
+    vector<vector<key_t> > GroupVector;
+    for (auto& W : Group)
         GroupVector.push_back(W);
     return GroupVector;
 }
 
 vector<vector<key_t> > orbits(const vector<vector<key_t> >& Perms, size_t N) {
-// Perms is a list of permutations of 0,...,N-1
-// We create the orbits of the permitation group generated by them. 
+    // Perms is a list of permutations of 0,...,N-1
+    // We create the orbits of the permitation group generated by them.
 
     vector<vector<key_t> > Orbits;
     if (Perms.size() == 0) {  // each element is its own orbit
         Orbits.reserve(N);
         for (size_t i = 0; i < N; ++i)
-            Orbits.push_back(vector<key_t>(1, i));
+            Orbits.push_back(vector<key_t>(1, static_cast<key_t>(i)));
         return Orbits;
     }
     vector<bool> InOrbit(N, false);
@@ -1039,7 +1333,7 @@ vector<vector<key_t> > orbits(const vector<vector<key_t> >& Perms, size_t N) {
         if (InOrbit[i])
             continue;
         vector<key_t> NewOrbit;
-        NewOrbit.push_back(i);
+        NewOrbit.push_back(static_cast<key_t>(i));
         InOrbit[i] = true;
         for (size_t j = 0; j < NewOrbit.size(); ++j) {
             for (const auto& Perm : Perms) {
@@ -1058,13 +1352,13 @@ vector<vector<key_t> > orbits(const vector<vector<key_t> >& Perms, size_t N) {
 }
 
 vector<vector<key_t> > convert_to_orbits(const vector<key_t>& raw_orbits) {
-// decomposes the orbit presentation of nauty into the standard form
+    // decomposes the orbit presentation of nauty into the standard form
     vector<key_t> key(raw_orbits.size());
     vector<vector<key_t> > orbits;
     for (key_t i = 0; i < raw_orbits.size(); ++i) {
         if (raw_orbits[i] == i) {
             orbits.push_back(vector<key_t>(1, i));
-            key[i] = orbits.size() - 1;
+            key[i] = static_cast<key_t>(orbits.size() - 1);
         }
         else {
             orbits[key[raw_orbits[i]]].push_back(i);
@@ -1074,7 +1368,7 @@ vector<vector<key_t> > convert_to_orbits(const vector<key_t>& raw_orbits) {
 }
 
 vector<vector<key_t> > cycle_decomposition(vector<key_t> perm, bool with_fixed_points) {
-    // computes the cacle decomposition of a permutation with or wothout fixed points
+    // computes the cacle decomposition of a permutation with or without fixed points
 
     vector<vector<key_t> > dec;
     vector<bool> in_cycle(perm.size(), false);
@@ -1084,14 +1378,14 @@ vector<vector<key_t> > cycle_decomposition(vector<key_t> perm, bool with_fixed_p
         if (perm[i] == i) {
             if (!with_fixed_points)
                 continue;
-            vector<key_t> cycle(1, i);
+            vector<key_t> cycle(1, static_cast<key_t>(i));
             in_cycle[i] = true;
             dec.push_back(cycle);
             continue;
         }
         in_cycle[i] = true;
-        key_t next = i;
-        vector<key_t> cycle(1, i);
+        key_t next = static_cast<key_t>(i);
+        vector<key_t> cycle(1, static_cast<key_t>(i));
         while (true) {
             next = perm[next];
             if (next == i)
